#ifndef Grid_H
#define Grid_H

#include "gridentity.h"
#include "cell.h"
#include <qstring.h>

class Grid
{
public:
	// Constructors
	Grid(QString);

	// Destructor
	~Grid();

	// Enums
	enum { NOTSET, EASY, AVERAGE, DIFFICULT};

	// Methods
	void Reset();
	void Solve();
	void Print();
	void Output(QString);
	void RotateLeft();
	void RotateRight();
	void FlipHorizontally();
	void FlipVertically();
	void SwapNumbers(int, int);

	// Properties
	int GetDifficultyLevel();
	void SetDifficultyLevel(int);
	QString GetGridName();
	void SetGridName(QString);
	bool IsSolved();
	int GetGridSize();

private:
	GridEntity* m_Squares;
	GridEntity* m_Rows;
	GridEntity* m_Columns;
	Cell* m_Cells;
	int m_DifficultyLevel;
	int m_intGridSize;
	bool m_blnSolved;
	QString m_strFileName;
	int m_intRecursionLevel;
	int* m_intCurrentGridValues;
	QString m_strGridName;

	void Initialise(QString);
	void InitialiseCells();
	bool Populate(QString);
	bool Populate(int*);
	int CalculateCellPossibleValues(int);
	int FindSingleCandidate(int);
	int FindHiddenSingle(int);
	bool FindNakedPair();
	bool FindNakedPair_Row(int, int, int, int, int);
	bool FindNakedPair_Column(int, int, int, int, int);
	//bool FindNakedPair_Square(int, int, int, int, int);
	bool CheckIfSolved();
	bool ValidateSolution();
};

#endif
