/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/
#include <qapplication.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qdom.h> 
#include <qfile.h>
#include <qstring.h>
#include <qtextstream.h> 
#include <qstatusbar.h>
#include <qradiobutton.h>
#include <qdir.h>
#include <qlineedit.h>
#include "grid.h"
#include "frmnewgrid.h"
#include "dlgabout.h"
#include "helper.h"

void frmSolver::init()
{
	TEMP_GRID_FILE = "newgridtemp.xml";
}

void frmSolver::fileNew()
{
	// Open the New Grid dialog
	frmNewGrid dialog(this);
	if(dialog.exec())
	{
		// Read the selected options
		int intLevel = 0;
		if(dialog.radEasy->isChecked())
			intLevel = 1;
		else
			intLevel = 3;

		m_strGridTitle = dialog.txtName->text();

		// Generate a new grid
		generateGrid(intLevel);
	}
}


void frmSolver::fileOpen()
{
	// Open the file dialog and open the selected grid file
	m_strFileName = QFileDialog::getOpenFileName("", "Sudoku Grid (*.xml)", this, "Sudoku Solver", "Choose a Sudoku Grid file" );
	if(m_strFileName != "")
		openGrid(0);
}


void frmSolver::fileSave()
{
	// Save the grid as XML
	QString strFileName = QFileDialog::getSaveFileName("", "Sudoku Grid (*.xml)", this, "Sudoku Solver", "Save Grid" );
	if(strFileName != "")
	{
		QDir dir(strFileName);
		dir.rename(m_strFileName, strFileName);
		m_strFileName = strFileName;
		fileSaveAction->setEnabled(false);
	}
}


void frmSolver::closeEvent( QCloseEvent * )
{
	fileExit();
}

void frmSolver::fileExit()
{
	// Remove temporary files
	QString strTempFile = "/tmp" + TEMP_GRID_FILE;
	QDir dir(strTempFile);
	if(dir.exists(strTempFile))
		dir.remove(strTempFile);
	
	strTempFile = "/tmp/gridtemp.htm";
	QDir dir2(strTempFile);
	if(dir2.exists(strTempFile))
		dir2.remove(strTempFile);

	// Exit the application
    QApplication::exit( 0 );
}

void frmSolver::helpAbout()
{
	// Open the About dialog
	dlgAbout dialog(this);
	dialog.exec();
}

void frmSolver::solve()
{
    // Create a new grid
	Grid* myGrid;
	myGrid = new Grid(m_strFileName);
	myGrid->Solve();

	if(!myGrid->IsSolved())
		QMessageBox::information(this, "Sudoku Solver", "Unable to solve this puzzle!");
	else
	{
		int intLevel = myGrid->GetDifficultyLevel();
	
		// Display the grid
		DisplayGrid("/tmp/" + TEMP_GRID_FILE, intLevel);
	
		// Enable the Solve button and menu option
		actionSolveAction->setEnabled(false);
		fileSaveAction->setEnabled(true);
		m_strFileName = "/tmp/" + TEMP_GRID_FILE;
	}

	delete myGrid;

}


void frmSolver::DisplayGrid(QString strFileName, int intLevel)
{
	// Open the xml file
	QDomDocument xmlDoc;
	QFile file(strFileName);
    if(!file.open(IO_ReadOnly))
	{
		QMessageBox::critical(0, "Sudoku Solver", "Couldn't open file '" + strFileName + "'.");
        return;
	}
    if (!xmlDoc.setContent(&file))
	{
        file.close();
		QMessageBox::critical(0, "Sudoku Solver", "Couldn't open file '" + strFileName + "'.");
        return;
    }
    file.close();

	// Get the grid size
    QDomElement docElem = xmlDoc.documentElement();
	QDomAttr xmlGridSize = xmlDoc.documentElement().attributeNode("size");
	int intGridSize = atoi(xmlGridSize.value());

	// Get the grid name
	QDomAttr xmlGridName = xmlDoc.documentElement().attributeNode("name");
	m_strGridTitle = xmlGridName.value();

	// Initialise array of values
	QString strGrid[intGridSize * intGridSize];

	// Read the contents and populate the array
	QDomNodeList xmlCellList = xmlDoc.elementsByTagName("cell");
	int intCellCount = int(xmlCellList.length());
	if(intCellCount > 0)
	{
		for(int i=0; i<intCellCount; i++)
		{
			QDomElement xmlCell = xmlCellList.item(i).toElement();
			QDomAttr xmlIndex = xmlCell.attributeNode("index");
			strGrid[atoi(xmlIndex.value())] = xmlCell.text();
		}
	}

	// Build the HTML
	QString strRating = GetRatingText(intLevel);
	QString strHTML = "<HTML><HEAD></HEAD><BODY>";
	strHTML += "<H3>Grid: " + m_strGridTitle + "</H3>";
	if(strRating != "")
		strHTML += "<H4>Rating: " + strRating + "</H4>";
	strHTML += "<TABLE BORDER=1>";
	for(int intRow=0; intRow<intGridSize; intRow++)
	{
		strHTML += "<TR>";
		for(int intCol=0; intCol<intGridSize; intCol++)
		{
			int intCellIndex = (intRow*intGridSize)+intCol;
			strHTML += "<TD ALIGN='center'>";
			if(strGrid[intCellIndex] == "")
				strHTML += "&nbsp;";
			else
				strHTML += "<FONT SIZE=5>" + strGrid[intCellIndex] + "</FONT>";
			strHTML += "</TD>";
		}
		strHTML += "</TR>";
	}
	strHTML += "</TABLE></BODY></HTML>";
	
	// Save to a temporary file and display it
	QString strTempFile = "/tmp/gridtemp.htm";
	QFile fGridTemp(strTempFile);
	if(!fGridTemp.open(IO_WriteOnly))
	{
		QMessageBox::critical(0, "Sudoku Solver", "Couldn't open file '" + strTempFile + "'.");
        return;
	}

	QTextStream out(&fGridTemp);
	out << strHTML;
	fGridTemp.close();

	qtbGrid->setSource(strTempFile);
	qtbGrid->reload();
}


QString frmSolver::GetRatingText( int intLevel )
{
	// Get text appropriate for difficulty level
	QString strText;
	switch(intLevel)
	{
		case 1:
			strText += "easy";
			break;
		case 2:
			strText += "average";
			break;
		case 3:
			strText += "difficult";
			break;
		default:
			strText += "";
	}

	return strText;
}

void frmSolver::openGrid(int intLevel)
{
	m_strGridTitle = "";

	// Enable the Solve button and menu option
	actionSolveAction->setEnabled(true);
	fileSaveAction->setEnabled(true);
 
     // Display the grid
	DisplayGrid(m_strFileName, intLevel);

}

void frmSolver::generateGrid(int intLevel)
{
	//===============================================================
	// Choose a random grid
	//===============================================================
	// Calculate how many grids of the required level are loaded
	QString strPath = QDir::currentDirPath() + "/grids";
	QDir dirGrids(strPath);
	QStringList slGrids;

	if(intLevel == 1)
		slGrids = dirGrids.entryList("e*.xml");
	else
		slGrids = dirGrids.entryList("d*.xml");

	int intGridCount = slGrids.count();

	// Choose one at random
	QString strGridNumber;
	strGridNumber.setNum(GenerateRandomInteger(intGridCount) + 1);
	if(intLevel == 1)
		m_strFileName = strPath + "/e" + strGridNumber + ".xml";
	else
		m_strFileName = strPath + "/d" + strGridNumber + ".xml";

	// Initialise the grid from the chosen file
	Grid* newGrid;
	newGrid = new Grid(m_strFileName);
	newGrid->SetGridName(m_strGridTitle);
	int intGridSize = newGrid->GetGridSize();

	//===============================================================
	// Choose random transformations
	//===============================================================

	if(GenerateRandomBoolean())
	{
		// Do number swaps
		int intNumberOfSwaps = GenerateRandomInteger(intGridSize) + 1;	
		int intFirstNumber = 0;
		int intSecondNumber = 0;
		for(int i=0; i < intNumberOfSwaps; i++)
		{
			do
			{
				intFirstNumber = GenerateRandomInteger(intGridSize) + 1;	
				intSecondNumber = GenerateRandomInteger(intGridSize) + 1;	
			} while(intFirstNumber == intSecondNumber);
	
			newGrid->SwapNumbers(intFirstNumber,intSecondNumber);
		}
	}

	// Rotate the grid (0 = no rotation, 1 = left, 2 = right)
	int intRotation = GenerateRandomInteger(3);
	if(intRotation == 1)
		newGrid->RotateLeft();
	else if(intRotation == 2)
		newGrid->RotateRight();

	// Flip the grid (0 = no flip, 1 = horizontal, 2 = vertical)
	intRotation = GenerateRandomInteger(3);
	if(intRotation == 1)
		newGrid->FlipHorizontally();
	else if(intRotation == 2)
		newGrid->FlipVertically();

	//===============================================================
	// Open and display the grid
	//===============================================================

	m_strFileName = "/tmp/" + TEMP_GRID_FILE;

	// Save the grid to a temporary file
	newGrid->Output(m_strFileName);
	delete newGrid;

	// Open the grid & display it
	openGrid(intLevel);
}
